@@ -1,6 +1,6 @@
 /* ncdu - NCurses Disk Usage 
   
-  Copyright (c) 2007-2008 Yoran Heling
+  Copyright (c) 2007-2009 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -23,33 +23,40 @@
 
 */
 
-#include "ncdu.h"
+#include "global.h"
+
+#include <ncurses.h>
+#include <string.h>
+
+
+int page, start;
+
 
 #define KEYS 14
-
 char *keys[KEYS*2] = {
 /*|----key----|  |----------------description----------------|*/
-      "up/down", "Cycle through the items",
-  "right/enter", "Open directory",
-         "left", "Previous directory",
+        "up, k", "Move cursor up",
+      "down, j", "Move cursor down",
+  "right/enter", "Open selected directory",
+   "left, <, h", "Open parent directory",
             "n", "Sort by name (ascending/descending)",
             "s", "Sort by size (ascending/descending)",
             "d", "Delete selected file or directory",
             "t", "Toggle dirs before files when sorting",
             "g", "Show percentage and/or graph",
-            "p", "Toggle between powers of 1000 and 1024",
             "a", "Toggle between apparent size and disk usage",
-            "h", "Show/hide hidden or excluded files",
+            "e", "Show/hide hidden or excluded files",
             "i", "Show information about selected item",
             "r", "Recalculate the current directory",
             "q", "Quit ncdu"
 };
 
 
-
-void drawHelp(int page, int start) {
+void help_draw() {
   int i, line;
 
+  browse_draw();
+
   nccreate(15, 60, "ncdu help");
   ncaddstr(13, 38, "Press q to continue");
 
@@ -89,14 +96,16 @@ void drawHelp(int page, int start) {
       ncaddch( 7, 4, '<');
       ncaddch( 8, 4, '>');
       ncaddch( 9, 4, '@');
-      ncaddch(10, 4, 'e');
+      ncaddch(10, 4, 'H');
+      ncaddch(11, 4, 'e');
       attroff(A_BOLD);
       ncaddstr( 5, 7, "An error occured while reading this directory");
       ncaddstr( 6, 7, "An error occured while reading a subdirectory");
       ncaddstr( 7, 7, "File or directory is excluded from the statistics");
       ncaddstr( 8, 7, "Directory was on an other filesystem");
       ncaddstr( 9, 7, "This is not a file nor a dir (symlink, socket, ...)");
-      ncaddstr(10, 7, "Empty directory");
+      ncaddstr(10, 7, "Same file was already counted (hard link)");
+      ncaddstr(11, 7, "Empty directory");
       break;
     case 3:
       /* Indeed, too much spare time */
@@ -145,54 +154,53 @@ void drawHelp(int page, int start) {
       ncaddstr(10, 16, "http://dev.yorhel.nl/ncdu/");
       break;
   }
-  refresh();
 }
 
 
-void showHelp(void) {
-  int p = 1, st = 0, ch;
-
-  drawHelp(p, st);
-  while((ch = getch())) {
-    switch(ch) {
-      case ERR:
-        break;
-      case '1':
-      case '2':
-      case '3':
-        p = ch-'0';
-        st = 0;
-        break;
-      case KEY_RIGHT:
-      case KEY_NPAGE:
-        if(++p > 3)
-          p = 3;
-        st = 0;
-        break;
-      case KEY_LEFT:
-      case KEY_PPAGE:
-        if(--p < 1)
-          p = 1;
-        st = 0;
-        break;
-      case KEY_DOWN:
-      case ' ':
-        if(st < KEYS-10)
-          st++;
-        break;
-      case KEY_UP:
-        if(st > 0)
-          st--;
-        break;
-      case KEY_RESIZE:
-        ncresize();
-        drawBrowser(0);
-        break;
-      default:
-        return;
-    }
-    drawHelp(p, st);
+int help_key(int ch) {
+  switch(ch) {
+    case '1':
+    case '2':
+    case '3':
+      page = ch-'0';
+      start = 0;
+      break;
+    case KEY_RIGHT:
+    case KEY_NPAGE:
+    case 'l':
+      if(++page > 3)
+        page = 3;
+      start = 0;
+      break;
+    case KEY_LEFT:
+    case KEY_PPAGE:
+    case 'h':
+      if(--page < 1)
+        page = 1;
+      start = 0;
+      break;
+    case KEY_DOWN:
+    case ' ':
+    case 'j':
+      if(start < KEYS-10)
+        start++;
+      break;
+    case KEY_UP:
+    case 'k':
+      if(start > 0)
+        start--;
+      break;
+    default:
+      pstate = ST_BROWSE;
   }
+  return 0;
+}
+
+
+void help_init() {
+  page = 1;
+  start = 0;
+  pstate = ST_HELP;
 }
 
 
