@@ -1,6 +1,6 @@
 /* ncdu - NCurses Disk Usage 
     
-  Copyright (c) 2007-2008 Yoran Heling
+  Copyright (c) 2007-2009 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -23,29 +23,79 @@
 
 */
 
-#include "ncdu.h"
+#include "global.h"
 
-/* check ncdu.h what these are for */
-struct dir *dat;
-int winrows, wincols;
-char sdir[PATH_MAX];
-int sflags, bflags, sdelay, bgraph;
-int subwinc, subwinr;
+#include <stdlib.h>
+#include <string.h>
+#include <errno.h>
+
+#include <unistd.h>
+#include <sys/time.h>
+#include <locale.h>
+
+int pstate;
+
+int min_rows = 17,
+    min_cols = 60;
+long update_delay = 100,
+     lastupdate = 999;
+
+
+void screen_draw() {
+  switch(pstate) {
+    case ST_CALC:   calc_draw();   break;
+    case ST_BROWSE: browse_draw(); break;
+    case ST_HELP:   help_draw();   break;
+    case ST_DEL:    delete_draw(); break;
+  }
+}
+
+
+/* wait:
+ *  -1: non-blocking, always draw screen
+ *   0: blocking wait for input and always draw screen
+ *   1: non-blocking, draw screen only if a configured delay has passed or after keypress
+ */
+int input_handle(int wait) {
+  int ch;
+  struct timeval tv;
+
+  nodelay(stdscr, wait?1:0);
+  if(wait != 1)
+    screen_draw();
+  else {
+    gettimeofday(&tv, (void *)NULL);
+    tv.tv_usec = (1000*(tv.tv_sec % 1000) + (tv.tv_usec / 1000)) / update_delay;
+    if(lastupdate != tv.tv_usec) {
+      screen_draw();
+      lastupdate = tv.tv_usec;
+    }
+  }
+  while((ch = getch()) != ERR) {
+    if(ch == KEY_RESIZE) {
+      if(ncresize(min_rows, min_cols))
+        min_rows = min_cols = 0;
+      screen_draw();
+      continue;
+    }
+    switch(pstate) {
+      case ST_CALC:   return calc_key(ch);
+      case ST_BROWSE: return browse_key(ch);
+      case ST_HELP:   return help_key(ch);
+      case ST_DEL:    return delete_key(ch);
+    }
+    screen_draw();
+  }
+  return 0;
+}
 
 
 /* parse command line */
-void parseCli(int argc, char **argv) {
-  int i, j;
+char *argv_parse(int argc, char **argv) {
+  int i, j, len;
+  char *dir = NULL;
 
- /* load defaults */
-  memset(sdir, 0, PATH_MAX);
-  getcwd(sdir, PATH_MAX);
-  sflags = 0;
-  sdelay = 100;
-  bflags = BF_SIZE | BF_DESC;
-  sdir[0] = '\0';
-
- /* read from commandline */
+  /* read from commandline */
   for(i=1; i<argc; i++) {
     if(argv[i][0] == '-') {
      /* flags requiring arguments */
@@ -56,18 +106,19 @@ void parseCli(int argc, char **argv) {
           exit(1);
         }
         else if(strcmp(argv[i], "--exclude") == 0)
-          addExclude(argv[++i]);
-        else if(addExcludeFile(argv[++i])) {
+          exclude_add(argv[++i]);
+        else if(exclude_addfile(argv[++i])) {
           printf("Can't open %s: %s\n", argv[i], strerror(errno));
           exit(1);
         }
         continue;
       }
      /* small flags */
-      for(j=1; j < strlen(argv[i]); j++)
+      len = strlen(argv[i]);
+      for(j=1; j<len; j++)
         switch(argv[i][j]) {
-          case 'x': sflags |= SF_SMFS; break;
-          case 'q': sdelay = 2000;     break;
+          case 'x': calc_smfs = 1; break;
+          case 'q': update_delay = 2000;     break;
           case '?':
           case 'h':
             printf("ncdu [-hqvx] [--exclude PATTERN] [-X FILE] directory\n\n");
@@ -85,40 +136,46 @@ void parseCli(int argc, char **argv) {
             printf("Unknown option: -%c\nSee '%s -h' for more information.\n", argv[i][j], argv[0]);
             exit(1);
         }
-    } else {
-      sdir[PATH_MAX - 1] = 0;
-      strncpy(sdir, argv[i], PATH_MAX);
-      if(sdir[PATH_MAX - 1] != 0)
-        sdir[0] = 0;
-    }
-  }
-  if(!sdir[0]) {
-    printf("Please specify a directory.\nSee '%s -h' for more information.\n", argv[0]);
-    exit(1);
+    } else
+      dir = argv[i];
   }
+  return dir;
 }
 
 
 /* main program */
 int main(int argc, char **argv) {
-  dat = NULL;
+  char *dir;
 
-  parseCli(argc, argv);
+  setlocale(LC_ALL, "");
+
+  if((dir = argv_parse(argc, argv)) == NULL)
+    dir = ".";
+
+  calc_init(dir, NULL);
 
   initscr();
   cbreak();
   noecho();
   curs_set(0);
   keypad(stdscr, TRUE);
-  ncresize();
-  
-  if((dat = showCalc(sdir)) != NULL)
-    showBrowser();
+  if(ncresize(min_rows, min_cols))
+    min_rows = min_cols = 0;
+
+  while(1) {
+    if(pstate == ST_CALC && calc_process())
+      break;
+    else if(pstate == ST_DEL)
+      delete_process();
+    else if(input_handle(0))
+      break;
+  }
 
   erase();
   refresh();
   endwin();
+  exclude_clear();
 
-  return(0);
+  return 0;
 }
 
